/*
 * Copyright (c) 2004 Michael Connor. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of FormLayoutBuilder or Michael Connor nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package org.mlc.swing.layout;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import java.awt.Component;
import java.awt.LayoutManager2;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer; 
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;

/**
 * This class acts as a surogate layout manager
 * for the JGoodies (www.jgoodies.com) FormLayout
 * manager.  This layout manager enables us to
 * associate names with components and then define
 * the constraints for the component elsewhere (like xml)
 *
 * @author Michael Connor
 */
public class ContainerLayout implements LayoutManager2
{
    List <String> rowSpecs = new ArrayList <String> ();
    List <String> columnSpecs = new ArrayList <String> ();
    LinkedHashMap <String, CellConstraints> componentConstraints = new LinkedHashMap <String, CellConstraints> ();
    Map <Component, String> componentsToNames = new HashMap <Component, String> ();    
    FormLayout formLayout;
    String name;
    

    public ContainerLayout (String name, String columnSpecs, String rowSpecs)
    {
        this.name = name;
        StringTokenizer cols = new StringTokenizer (columnSpecs, ",", false);
        StringTokenizer rows = new StringTokenizer (rowSpecs, ",", false);
        
        while (cols.hasMoreTokens())
            this.columnSpecs.add (cols.nextToken());
        
        while (rows.hasMoreTokens())
            this.rowSpecs.add (rows.nextToken());
        
        formLayout = new FormLayout (columnSpecs, rowSpecs);
        
    }        
    
    private void buildLayout ()
    throws IllegalArgumentException
    {
        formLayout = new FormLayout(delimit(columnSpecs) , delimit(rowSpecs));
    
        //System.out.println("New layout has row count " + formLayout.getRowCount());
        // now we have to add all of the compenents to the new form
        for ( Iterator i = componentsToNames.keySet().iterator(); i.hasNext(); )
        {
            Component component = (Component) i.next();
            String componentName = componentsToNames.get(component);
            CellConstraints constraints = componentConstraints.get(componentName);
            formLayout.addLayoutComponent(component, constraints);
        }
    }
    
    private String delimit(List <String> values)
    {
        StringBuffer buffer = new StringBuffer();
        
        for ( int index = 0; index < values.size(); index++ )
        {
            buffer.append(values.get(index));
            if ( index < values.size() - 1 )
                buffer.append(",");
        }
        
        return buffer.toString();
    }    
    
    protected Map <Component, String> getComponentsToNames ()
    {
        return Collections.unmodifiableMap(componentsToNames);
    }
    
    /**
     * Registers the value of the name property
     *
     * @param name The value of the property
     */
    public void setName (String name)
    {
        this.name = name;
    }
    
    /**
     * Returns the value of the name property
     *
     * @return The value
     */
    public String getName ()
    {
        return this.name;
    }
   
    public void setCellConstraints (String componentName, CellConstraints constraints)
    {
        componentConstraints.put (name, constraints);

        for ( Iterator i = componentsToNames.keySet().iterator(); i.hasNext(); )
        {
            Component component = (Component) i.next();
            String thisName = componentsToNames.get (component);
            if ( thisName.equals (componentName) )
            {
                formLayout.setConstraints(component, constraints);
                break;
            }
        }
    }
       
    public LinkedHashMap <String, CellConstraints> getCellConstraints ()
    {
        return componentConstraints;
    }
    
    public String getColumnSpecsString ()
    {
        StringBuffer buffer = new StringBuffer();
        for ( Iterator i = columnSpecs.iterator(); i.hasNext(); )
        {
            buffer.append (i.next());
            if ( i.hasNext() )
                buffer.append (",");
        }
        
        return buffer.toString();
    }    
    
    public String getRowSpecsString ()
    {
        StringBuffer buffer = new StringBuffer();
        for ( Iterator i = rowSpecs.iterator(); i.hasNext(); )
        {
            buffer.append (i.next());
            if ( i.hasNext() )
                buffer.append (",");
        }
        
        return buffer.toString();
    }
    
    public int getRowCount ()
    {
        return rowSpecs.size();
    }
    
    public int getColumnCount()
    {
        return columnSpecs.size();
    }
    
    public List <String> getRowSpecs()
    {
        return this.rowSpecs;
    }
    
    public List <String> getColumnSpecs()
    {
        return this.columnSpecs;
    }
    
    public void constraintsChanged (String name, CellConstraints constraints)
    {
        componentConstraints.put (name, constraints);
    }
    
    public CellConstraints getCellConstraints (String name)
    {
        return componentConstraints.get(name);
    }
    
    public void addCellConstraints (String name, CellConstraints constraints)
    {
        componentConstraints.put (name, constraints);
    }
    
    public CellConstraints removeCellConstraints (String name)
    {
        CellConstraints constraints = (CellConstraints) componentConstraints.remove (name);
        return constraints;
    }
    
    public void addColumnSpec (String columnSpec)
    throws IllegalArgumentException
    {
        columnSpecs.add (columnSpec);
        buildLayout();
    }

    public String getRowSpec (int index)
    {
        return rowSpecs.get (index);
    }    
    
    public String getColumnSpec (int index)
    {
        return columnSpecs.get (index);
    }
    
    public void setRowSpec (int index, String rowSpec)
    throws IllegalArgumentException
    {
        rowSpecs.set (index, rowSpec);
        buildLayout();
    }
    
    public void setColumnSpec (int index, String columnSpec)
    throws IllegalArgumentException
    {
        columnSpecs.set (index, columnSpec);
        buildLayout();
    }
    
    public void addRowSpec (String rowSpec)
    throws IllegalArgumentException
    {
        System.out.println("adding row spec " + rowSpecs.size());
        rowSpecs.add (rowSpec);
        System.out.println("done adding row spec " + rowSpecs.size());
        buildLayout();
    }

    public String removeRowSpec (int index)
    {
        String rowSpec = (String) rowSpecs.remove(index);
        try {
            buildLayout();
        } catch (IllegalArgumentException e)
        {
            throw new RuntimeException (e);
        }
        
        return rowSpec;
    }    
    
    public String removeColumnSpec (int index)
    {
        String spec = columnSpecs.remove(index);
        try {
            buildLayout();
        } catch (IllegalArgumentException e)
        {
            throw new RuntimeException (e);
        }

        return spec; 
    }
        
    public void addRowSpec (int index, String rowSpec)
    throws IllegalArgumentException
    {
        System.out.println("adding row spec " + rowSpecs.size());
        rowSpecs.add(index, rowSpec);
        System.out.println("done adding row spec " + rowSpecs.size());                
        buildLayout();
    }
    
    public void addColumnSpec (int index, String columnSpec)
    throws IllegalArgumentException
    {
        columnSpecs.add(index, columnSpec);
        buildLayout();
    }

    /* the following methods realize the LayoutManager interface */
   
    public String getComponentName (Component component) {
        return componentsToNames.get (component);
    }
    
    /**
     * Returns the component with the
     * given name or null if not found
     */
    public Component getComponentByName(String name) {
        for ( Component component : componentsToNames.keySet() ) {
            String testName = componentsToNames.get(component);
            if ( testName.equals(name) )
                return component;
        }
        return null;
    }
    
    public CellConstraints getComponentConstraints (Component component)
    {
        String name = componentsToNames.get (component);
        if ( name == null )
            throw new RuntimeException ("Unable to find name for component " + component);
        return componentConstraints.get (name);
    }
    
    // interface for LayoutManager2
    public void addLayoutComponent(String name, java.awt.Component comp)
    {
        throw new RuntimeException ("This method should not be called.  Call addLayoutComponent (Component, Object) instead");
    }
    
    public float getLayoutAlignmentX(java.awt.Container target)
    {
        return formLayout.getLayoutAlignmentX(target);
    }
    
    public float getLayoutAlignmentY(java.awt.Container target)
    {
        return formLayout.getLayoutAlignmentY(target);
    }
    
    public void invalidateLayout(java.awt.Container target)
    {
        formLayout.invalidateLayout(target);
    }
    
    public void layoutContainer(java.awt.Container parent)
    {
        formLayout.layoutContainer(parent);
    }
    
    public java.awt.Dimension maximumLayoutSize(java.awt.Container target)
    {
        return formLayout.maximumLayoutSize(target);
    }
    
    public java.awt.Dimension minimumLayoutSize(java.awt.Container parent)
    {
        return formLayout.minimumLayoutSize(parent);
    }
    
    public java.awt.Dimension preferredLayoutSize(java.awt.Container parent)
    {
        return formLayout.preferredLayoutSize(parent);
    }
    
    public void removeLayoutComponent(java.awt.Component comp)
    {
        String componentName = componentsToNames.get (comp);
        componentsToNames.remove (comp);
        componentConstraints.remove(componentName);
        formLayout.removeLayoutComponent(comp);
    }        
    
    public void addLayoutComponent(java.awt.Component comp, Object constraints)
    {
        if ( ! (constraints instanceof String) )
            throw new RuntimeException ("The constraints must be a String name which should reference a CellConstraints entry in the xml file");
        String componentName = (String) constraints;
        CellConstraints cellConstraints = componentConstraints.get (componentName);
        if ( cellConstraints == null )
        {
            System.out.println("Warning : " + componentName + " was added without constraints");
            cellConstraints = new CellConstraints();
            componentConstraints.put (componentName, cellConstraints);      
            comp.setVisible(false);
        }
//            // a set of fields and labels typically has four columns, a label column, gutter column, field column, gutter column
//            List<LocationScore> scoreList = new ArrayList<LocationScore>();
//            for ( int rowIndex = 1; rowIndex <= formLayout.getRowCount(); rowIndex++ )
//            {
//                for ( int columnIndex = 1; columnIndex <= formLayout.getColumnCount(); columnIndex++ )
//                {
//                    int resolvedColumn = columnIndex % 4;
//                    int score = 0;
//                    int width = 1;
//
//                    if ( comp instanceof JLabel && resolvedColumn == 1 )
//                        score++;
//                    else if ( (comp instanceof JScrollPane || 
//                               comp instanceof JPanel ||
//                               comp instanceof JSplitPane ||
//                               comp instanceof JTabbedPane )
//                              && columnIndex == 1 )
//                    {
//                        score++;
//                        width = formLayout.getColumnCount() - columnIndex + 1;
//                    } else if ( resolvedColumn == 3 )
//                        score++;
//                    // it's better for all components to be in odd columns
//                    if ( rowIndex % 2 == 1 )
//                        score++;
//                    
//                    // we need to see if any other components occupy this place.
//                    // more than anything else we want to make sure components
//                    // are spread out and not on top of each other.
//                    // here we'll build a matrix of all occupied cells taking
//                    // the width of the components into consideration
//                    boolean[][] cellOccupied = new boolean[formLayout.getColumnCount() + 1][formLayout.getRowCount() + 1]; // extra is for 1 based system
//                    
//                    for ( Component existingComponent : componentsToNames.keySet() )  {
//                        CellConstraints c = formLayout.getConstraints(existingComponent);
//                        for ( int widthIndex = 0; widthIndex < c.gridWidth; widthIndex++ )
//                            cellOccupied[c.gridX + widthIndex][c.gridY] = true;
//                    }
//                    
//                    // now we check to see if there are any collisions
//                    for ( int widthIndex = 0; widthIndex < width; widthIndex++ ) {
//                        if ( cellOccupied[columnIndex + widthIndex][rowIndex] ) {
//                            score = score - 2;
//                            break;
//                        }
//                    }
//                    
//                    LocationScore locationScore = new LocationScore(score, rowIndex, columnIndex, width);
//                    int insertionIndex = Collections.binarySearch(scoreList, locationScore);
//                    if ( insertionIndex < 0 ) // if no score that is the same exists...
//                        scoreList.add( insertionIndex * -1 - 1, locationScore);
//                }
//            }
//            
//            //Collections.sort(scoreList);
//            LocationScore bestScore = scoreList.get(scoreList.size() - 1);
//            cellConstraints = new CellConstraints(bestScore.col, bestScore.row);
//            cellConstraints.gridWidth = bestScore.width;
//        }
        
        componentsToNames.put (comp, componentName);
        formLayout.addLayoutComponent(comp, cellConstraints);
    }        
    
    private class LocationScore implements Comparable<LocationScore>
    {
        public int score;
        public int row;
        public int col;
        public int width;
        
        public LocationScore(int score, int row, int col, int width)
        {
            this.score = score;
            this.row = row;
            this.col = col;
            this.width = width;
        }
        
        public int compareTo(LocationScore testScore)
        {
            return this.score < testScore.score ? -1 : this.score > testScore.score ? 1 : 0;
        }
    }
    
    public static void main (String[] args)
    {
        System.out.println(5 % 4);
    }
    
}
